@@ -159,6 +159,7 @@ impl Taos {
 mod test {
     use crate::test::taos;
     use crate::*;
+    use proc_test_catalog::test_catalogue;
 
     async fn stmt_test(db: &str, ty: &str, value: Field) -> Result<(), Error> {
         let taos = taos()?;
@@ -184,28 +185,106 @@ mod test {
         taos.exec(format!("drop database {}", db)).await?;
         Ok(())
     }
+
+    macro_rules! _test_column_null {
+        ($ty:ty, $v:expr) => {
+            paste::paste! {
+                #[tokio::test]
+                #[test_catalogue()]
+                #[doc = "Test bind null to type " $ty]
+                async fn [<null_ $ty:snake>]() -> Result<(), Error> {
+                    let db = stdext::function_name!()
+                        .replace("::{{closure}}", "")
+                        .replace("::", "_");
+                    stmt_test(&db, $v, Field::Null).await
+                }
+            }
+        };
+    }
+    _test_column_null!(bool, "bool");
+    _test_column_null!(tinyint, "tinyint");
+    _test_column_null!(smallint, "smallint");
+    _test_column_null!(int, "int");
+    _test_column_null!(bigint, "bigint");
+    _test_column_null!(utinyint, "tinyint unsigned");
+    _test_column_null!(usmallint, "smallint unsigned");
+    _test_column_null!(uint, "int unsigned");
+    _test_column_null!(ubigint, "bigint unsigned");
+    _test_column_null!(timestamp, "timestamp");
+    _test_column_null!(float, "float");
+    _test_column_null!(double, "double");
+    _test_column_null!(binary, "binary(10)");
+    _test_column_null!(nchar, "nchar(10)");
+    #[should_panic]
     #[tokio::test]
-    async fn bool_null() -> Result<(), Error> {
+    #[test_catalogue()]
+    /// Test bind null to json tag, should panic because json is not supported in cols
+    async fn null_json() -> () {
         let db = stdext::function_name!()
             .replace("::{{closure}}", "")
             .replace("::", "_");
-        stmt_test(&db, "bool", Field::Null).await
+        stmt_test(&db, "json", Field::Null).await.unwrap()
     }
-    #[tokio::test]
-    async fn int_null() -> Result<(), Error> {
-        let db = stdext::function_name!()
-            .replace("::{{closure}}", "")
-            .replace("::", "_");
-        stmt_test(&db, "int", Field::Null).await
+
+    async fn stmt_tag(db: &str, ty: &str, tag: Field) -> Result<(), Error> {
+        let taos = taos()?;
+        println!("test {} using {}", ty, db);
+        taos.exec(format!("drop database if exists {}", db)).await?;
+        taos.exec(format!("create database if not exists {} keep 36500", db))
+            .await?;
+        taos.exec(format!("use {}", db)).await?;
+        taos.exec(format!(
+            "create table if not exists stb0 (ts timestamp, v int) tags (n {})",
+            ty
+        ))
+        .await?;
+        let mut stmt = taos.stmt("insert into ? using stb0 tags(?) values(?,?)")?;
+        stmt.set_tbname_tags("tb0", [&tag])?;
+        assert!(stmt.is_insert());
+        assert_eq!(stmt.num_params(), 2);
+        let ts = Field::Timestamp(Timestamp::now());
+        stmt.bind(vec![ts, Field::Null].iter())?;
+        let _ = stmt.execute()?;
+        let res = taos.query("select n from tb0").await?;
+        assert_eq!(tag, res.rows[0][0]);
+        taos.exec(format!("drop database {}", db)).await?;
+        Ok(())
     }
-    #[tokio::test]
-    async fn float_null() -> Result<(), Error> {
-        let db = stdext::function_name!()
-            .replace("::{{closure}}", "")
-            .replace("::", "_");
-        stmt_test(&db, "float", Field::Null).await
+    macro_rules! _test_tag_null {
+        ($ty:ty, $v:expr) => {
+            paste::paste! {
+                #[tokio::test]
+                #[test_catalogue()]
+                #[doc = "Test bind null to type " $ty]
+                async fn [<null_tag_ $ty:snake>]() -> Result<(), Error> {
+                    let db = stdext::function_name!()
+                        .replace("::{{closure}}", "")
+                        .replace("::", "_")
+                        .replace("libtaos_", "");
+                    stmt_tag(&db, $v, Field::Null).await
+                }
+            }
+        };
     }
+    _test_tag_null!(bool, "bool");
+    _test_tag_null!(tinyint, "tinyint");
+    _test_tag_null!(smallint, "smallint");
+    _test_tag_null!(int, "int");
+    _test_tag_null!(bigint, "bigint");
+    _test_tag_null!(utinyint, "tinyint unsigned");
+    _test_tag_null!(usmallint, "smallint unsigned");
+    _test_tag_null!(uint, "int unsigned");
+    _test_tag_null!(ubigint, "bigint unsigned");
+    _test_tag_null!(timestamp, "timestamp");
+    _test_tag_null!(float, "float");
+    _test_tag_null!(double, "double");
+    _test_tag_null!(binary, "binary(10)");
+    _test_tag_null!(nchar, "nchar(10)");
+    // set null in json tag is currently abort taosd. see TD-12452.
+    // _test_tag_null!(json, "json");
     #[tokio::test]
+    #[test_catalogue()]
+    /// Test STMT inserting with bool values.
     async fn bool() -> Result<(), Error> {
         let db = stdext::function_name!()
             .replace("::{{closure}}", "")
@@ -213,6 +292,8 @@ mod test {
         stmt_test(&db, "bool", Field::Bool(true)).await
     }
     #[tokio::test]
+    #[test_catalogue()]
+    /// Test STMT inserting with tiny int values.
     async fn tinyint() -> Result<(), Error> {
         let db = stdext::function_name!()
             .replace("::{{closure}}", "")
@@ -220,6 +301,8 @@ mod test {
         stmt_test(&db, "tinyint", Field::TinyInt(-0x7f)).await
     }
     #[tokio::test]
+    #[test_catalogue()]
+    /// Test STMT inserting with small int values.
     async fn smallint() -> Result<(), Error> {
         let db = stdext::function_name!()
             .replace("::{{closure}}", "")
@@ -227,6 +310,8 @@ mod test {
         stmt_test(&db, "smallint", Field::SmallInt(0x7fff)).await
     }
     #[tokio::test]
+    #[test_catalogue()]
+    /// Test STMT inserting with int values.
     async fn int() -> Result<(), Error> {
         let db = stdext::function_name!()
             .replace("::{{closure}}", "")
@@ -234,6 +319,8 @@ mod test {
         stmt_test(&db, "int", Field::Int(0x7fffffff)).await
     }
     #[tokio::test]
+    #[test_catalogue()]
+    /// Test STMT inserting with bigint values.
     async fn bigint() -> Result<(), Error> {
         let db = stdext::function_name!()
             .replace("::{{closure}}", "")
@@ -241,6 +328,8 @@ mod test {
         stmt_test(&db, "bigint", Field::BigInt(0x7fffffff_ffffffff)).await
     }
     #[tokio::test]
+    #[test_catalogue()]
+    /// Test STMT inserting with unsigned tinyint values.
     async fn utinyint() -> Result<(), Error> {
         let db = stdext::function_name!()
             .replace("::{{closure}}", "")
@@ -248,6 +337,8 @@ mod test {
         stmt_test(&db, "tinyint unsigned", Field::UTinyInt(0)).await
     }
     #[tokio::test]
+    #[test_catalogue()]
+    /// Test STMT inserting with unsigned smallint values.
     async fn usmallint() -> Result<(), Error> {
         let db = stdext::function_name!()
             .replace("::{{closure}}", "")
@@ -255,6 +346,8 @@ mod test {
         stmt_test(&db, "smallint unsigned", Field::USmallInt(1)).await
     }
     #[tokio::test]
+    #[test_catalogue()]
+    /// Test STMT inserting with unsigned int values.
     async fn uint() -> Result<(), Error> {
         let db = stdext::function_name!()
             .replace("::{{closure}}", "")
@@ -262,6 +355,8 @@ mod test {
         stmt_test(&db, "int unsigned", Field::UInt(1)).await
     }
     #[tokio::test]
+    #[test_catalogue()]
+    /// Test STMT inserting with unsigned bigint values.
     async fn ubigint() -> Result<(), Error> {
         let db = stdext::function_name!()
             .replace("::{{closure}}", "")
@@ -269,6 +364,8 @@ mod test {
         stmt_test(&db, "bigint unsigned", Field::UBigInt(1)).await
     }
     #[tokio::test]
+    #[test_catalogue()]
+    /// Test STMT inserting with float values.
     async fn float() -> Result<(), Error> {
         let db = stdext::function_name!()
             .replace("::{{closure}}", "")
@@ -276,6 +373,8 @@ mod test {
         stmt_test(&db, "float", Field::Float(1.0)).await
     }
     #[tokio::test]
+    #[test_catalogue()]
+    /// Test STMT inserting with double values.
     async fn double() -> Result<(), Error> {
         let db = stdext::function_name!()
             .replace("::{{closure}}", "")
@@ -283,6 +382,8 @@ mod test {
         stmt_test(&db, "double", Field::Double(1.0)).await
     }
     #[tokio::test]
+    #[test_catalogue()]
+    /// Test STMT inserting with binary values.
     async fn binary() -> Result<(), Error> {
         let db = stdext::function_name!()
             .replace("::{{closure}}", "")
@@ -291,6 +392,8 @@ mod test {
         stmt_test(&db, "binary(10)", v).await
     }
     #[tokio::test]
+    #[test_catalogue()]
+    /// Test STMT inserting with nchar(unicode) values.
     async fn nchar() -> Result<(), Error> {
         let db = stdext::function_name!()
             .replace("::{{closure}}", "")
@@ -298,8 +401,42 @@ mod test {
         let v = Field::NChar("一二三四五六七八九十".into());
         stmt_test(&db, "nchar(10)", v).await
     }
+    #[tokio::test]
+    #[test_catalogue()]
+    /// Test STMT inserting with json values.
+    async fn json() -> Result<(), Error> {
+        let db = stdext::function_name!()
+            .replace("::{{closure}}", "")
+            .replace("::", "_");
+        let v = Field::Json(serde_json::from_str("{\"tag1\":\"一二三四五六七八九十\"}").unwrap());
+
+        let taos = taos()?;
+        println!("test json using {}", db);
+        taos.exec(format!("drop database if exists {}", db)).await?;
+        taos.exec(format!("create database if not exists {} keep 36500", db))
+            .await?;
+        taos.exec(format!("use {}", db)).await?;
+        taos.exec("create stable if not exists stb0 (ts timestamp, n int) tags(j json)")
+            .await?;
+        let mut stmt = taos.stmt("insert into ? using stb0 tags(?) values(?,?)")?;
+        println!("set tbname with tags");
+        stmt.set_tbname_tags("tb0", [&v])?;
+        println!("bind values");
+        assert!(stmt.is_insert());
+        assert_eq!(stmt.num_params(), 2);
+        let ts = Field::Timestamp(Timestamp::now());
+        stmt.bind(vec![ts, Field::Int(3)].iter())?;
+        let _ = stmt.execute()?;
+        let res = taos.query("select j from stb0").await?;
+        let row = res.rows.iter().next().unwrap();
+        assert_eq!(&v, row.iter().next().unwrap());
+        taos.exec(format!("drop database {}", db)).await?;
+        Ok(())
+    }
 
     #[tokio::test]
+    #[test_catalogue()]
+    /// Test STMT inserting with all types of values.
     async fn all_types() -> Result<(), Error> {
         let taos = taos()?;
         let db = stdext::function_name!()
@@ -343,11 +480,47 @@ mod test {
         Ok(())
     }
     #[tokio::test]
+    #[test_catalogue()]
+    /// Test STMT set tbname with upper-case stable, see jira TD-12977
+    async fn test_uppercase_tbname() -> Result<(), Error> {
+        let db = "uppercase_test";
+        let taos = taos()?;
+        taos.exec(format!("drop database if exists {db}")).await?;
+        taos.exec(format!("create database {db}")).await?;
+        taos.exec(format!("use {db}")).await?;
+        taos.exec(format!("create stable STB(ts timestamp, n int) tags(b int)")).await?;
+        let mut stmt = taos.stmt("insert into ? using STB tags(?) values(?, ?)")?;
+
+        stmt.set_tbname_tags("tb0", [0i32])?;
+        // stmt.bind(&[0i32])?;
+        let values = vec![Field::Timestamp(Timestamp::now()), Field::Int(10)];
+        stmt.bind(&values)?;
+
+        assert!(stmt.is_insert());
+        assert_eq!(stmt.num_params(), 2);
+
+        let _ = stmt.execute()?;
+        const LIMIT: i64 = 100;
+
+        for i in 1..LIMIT {
+            stmt.set_tbname_tags(format!("tb{}", i), &[2i32])?;
+            stmt.bind(&values)?;
+        }
+        let _ = stmt.execute()?;
+        let res = taos.query("select count(*) as count from stb").await?;
+        assert_eq!(res.rows[0][0], Field::BigInt(LIMIT));
+        taos.exec(format!("drop database {}", db)).await?;
+
+        Ok(())
+    }
+    #[tokio::test]
+    #[test_catalogue()]
+    /// Test STMT API insertion with tags
     async fn test_stmt_tags() -> Result<(), Error> {
         let db = stdext::function_name!()
             .replace("::{{closure}}", "")
             .replace("::", "_");
-        println!("{}", db);
+        println!("{:?}", db);
         let taos = taos()?;
         taos.exec(format!("drop database if exists {}", db)).await?;
         taos.exec(format!("create database if not exists {} keep 36500", db))
@@ -356,43 +529,26 @@ mod test {
         taos.exec("create table if not exists stb (ts timestamp, n int) tags(b int)")
             .await?;
 
-        let mut stmt = taos.stmt("insert into ? using stb tags(?) values(now,?)")?;
+        let mut stmt = taos.stmt("insert into ? using stb tags(?) values(?, ?)")?;
 
         stmt.set_tbname_tags("tb0", [0i32])?;
-        stmt.bind(&[0i32])?;
+        // stmt.bind(&[0i32])?;
+        let values = vec![Field::Timestamp(Timestamp::now()), Field::Int(10)];
+        stmt.bind(&values)?;
+
         assert!(stmt.is_insert());
-        assert_eq!(stmt.num_params(), 1);
-
-        stmt.set_tbname_tags("tb1", &[1i32])?;
-        stmt.bind(&[&1i32])?;
-
-        stmt.set_tbname_tags("tb2", &[2i32])?;
-        stmt.bind([&2i32])?;
-
-        stmt.set_tbname_tags("tb3", &[3i32])?;
-        stmt.bind([3i32])?;
-
-        stmt.set_tbname_tags("tb4", &[4i32])?;
-        stmt.bind([Field::Int(4)])?;
-
-        stmt.set_tbname_tags("tb5", &[5i32])?;
-        stmt.bind([&Field::Int(5)])?;
-
-        stmt.set_tbname_tags("tb6", &[6i32])?;
-        stmt.bind(&[Field::Int(6)])?;
-
-        stmt.set_tbname_tags("tb7", &[7i32])?;
-        stmt.bind(&[&Field::Int(5)])?;
-
-        stmt.set_tbname_tags("tb8", &[8i32])?;
-        stmt.bind(vec![Field::Int(8)])?;
-
-        stmt.set_tbname_tags("tb9", &[9i32])?;
-        stmt.bind(vec![&Field::Int(9)])?;
+        assert_eq!(stmt.num_params(), 2);
 
+        let _ = stmt.execute()?;
+        const LIMIT: i64 = 100;
+
+        for i in 1..LIMIT {
+            stmt.set_tbname_tags(format!("tb{}", i), &[2i32])?;
+            stmt.bind(&values)?;
+        }
         let _ = stmt.execute()?;
         let res = taos.query("select count(*) as count from stb").await?;
-        assert_eq!(res.rows[0][0], Field::BigInt(10));
+        assert_eq!(res.rows[0][0], Field::BigInt(LIMIT));
         taos.exec(format!("drop database {}", db)).await?;
         Ok(())
     }
